/*
  This source is part of the Illumination library
  Copyright (C) 2004  Tim Teulings

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include <Lum/Array.h>

#include <Lum/Space.h>

namespace Lum {

  Array::Array()
  {
    horizSpace=false;
    vertSpace=false;
  }

  void Array::SetHorizontalCount(size_t count)
  {
    horiz=count;
    vert=0;
  }

  void Array::SetVerticalCount(size_t count)
  {
    horiz=0;
    vert=count;
  }

  void Array::SetSpace(bool horiz, bool vert)
  {
    horizSpace=horiz;
    vertSpace=vert;
  }

  static int Count(int zahl, int div)
  {
    if (zahl%div!=0) {
      return (zahl/div)+1;
    }
    else {
      return (zahl/div);
    }
  }

  Array* Array::AddEmpty()
  {
    Add(new VSpace());

    return this;
  }

  void Array::CalcSize()
  {
    std::list<Object*>::iterator iter;
    size_t                       mw,mh,xSize,ySize;

    sWidth=0;
    sHeight=0;
    mw=0;
    mh=0;

    iter=list.begin();
    while (iter!=list.end()) {
      (*iter)->CalcSize();

      sWidth=std::max(sWidth,(*iter)->GetOWidth());
      sHeight=std::max(sHeight,(*iter)->GetOHeight());
      mw=std::max(mw,(*iter)->GetOMinWidth());
      mh=std::max(mh,(*iter)->GetOMinHeight());

      ++iter;
    }

    if (horiz!=0) {
      if (horizSpace) {
        ySize=(horiz-1)*OS::display->GetSpaceHorizontal(OS::Display::spaceInterObject);
      }
      else {
        ySize=0;
      }
      width=horiz*sWidth+ySize;
      minWidth=horiz*mw+ySize;

      xSize=Count(list.size(),horiz);
      if (vertSpace) {
        ySize=(xSize-1)*OS::display->GetSpaceVertical(OS::Display::spaceInterObject);
      }
      else {
        ySize=0;
      }
      height=xSize*sHeight+ySize;
      minHeight=xSize*mh+ySize;
    }
    else {
      if (vertSpace) {
        ySize=(vert-1)*OS::display->GetSpaceVertical(OS::Display::spaceInterObject);
      }
      else {
        ySize=0;
      }
      height=vert*sHeight+ySize;
      minHeight=vert*mh+ySize;

      xSize=Count(list.size(),vert);
      if (horizSpace) {
        ySize=(xSize-1)*OS::display->GetSpaceHorizontal(OS::Display::spaceInterObject);
      }
      else {
        ySize=0;
      }
      width=xSize*sWidth+ySize;
      minWidth=xSize*mw+ySize;
    }

    List::CalcSize();
  }

  void Array::Layout()
  {
    std::list<Object*>::iterator iter;
    int                          hCount,vCount,xp,yp,h,w,xPos,yPos;

    if (horiz!=0) {
      hCount=horiz;
      vCount=Count(list.size(),horiz);
    }
    else {
      hCount=Count(list.size(),vert);
      vCount=vert;
    }

    if (horizSpace) {
      h=(hCount-1)*OS::display->GetSpaceHorizontal(OS::Display::spaceInterObject);
    }
    else {
      h=0;
    }
    sWidth=(width-h) / hCount;

    if (vertSpace) {
      w=(vCount-1)*OS::display->GetSpaceVertical(OS::Display::spaceInterObject);
    }
    else {
      w=0;
    }
    sHeight=(height-w) / vCount;

    iter=list.begin();
    while (iter!=list.end()) {
      (*iter)->Resize(sWidth,sHeight);
      ++iter;
    }

    xPos=1;
    yPos=1;
    xp=x;
    yp=y;

    iter=list.begin();
    while (iter!=list.end()) {
      (*iter)->Move(xp + (sWidth-(*iter)->GetOWidth()) / 2,
                    yp + (sHeight-(*iter)->GetOHeight()) / 2);

      if ((xPos==hCount) && (vCount!=yPos)) {
        if (vertSpace) {
          yp+=OS::display->GetSpaceVertical(OS::Display::spaceInterObject);
        }
        yp+=sHeight;
        yPos++;
        xPos=1;
        xp=x;
      }
      else if (hCount!=xPos) {
        if (horizSpace) {
          xp+=OS::display->GetSpaceHorizontal(OS::Display::spaceInterObject);
        }
        xp+=sWidth;
        xPos++;
      }

      ++iter;
    }

    List::Layout();
  }
}
